==> Mudlib-CD.00.28.tar.gz: lib/std/player/cmd_sec.c <==
/*
  player/cmd_sec.c

  This is a subpart of player_sec.c

  Some standard commands that should always exist are defined here.
  This is also the place for the quicktyper command hook.

  This file is inherited by player_sec.c

*/
#include "/sys/stdproperties.h"
#include "/secure/std.h"
#include "/sys/macros.h"

/*
 * Prototypes
 */
void save_me(int value_items);
int compute_values(object *ob_list);
void compute_auto_str();
void compute_recover_str(int verb);
string show_age();
nomask int check_recover_loc();


static 	object	cmd_modifhook;		/* Object to use for modif. of cmds */

static nomask void
cmd_sec_reset()
{
    add_action("quit", "quit");
    add_action("save_character", "save");
    add_action("toggle_brief", "brief");
    add_action("change_password", "password");
    add_action("set_email", "email");
    call_out("autosave_me", 5 * 60);
}

/*
 * Function name:   set_cmdmodify
 * Description:	    Decide what object to use for command modification. The
 *                  object should define the function modify_command(str).
 *                  If an object is set, commands that are not starting with
 *                  a '$' will be passed to that objects modifycmd function.
 * Arguments:       ob: object that defines the command modification function
 */
public void
set_cmdmodify(object ob)
{
    if (geteuid(ob) != geteuid(this_object()) && geteuid(ob) !=
		SECURITY->get_root_uid())
	return;

    cmd_modifhook = ob;
}

/*
 * Function name:   query_cmdmodify
 * Description:	    Returns the object that is used for command modification
 * Returns:         0 if no modify object was defined,
 *                  the modify object otherwise
 */
public object
query_cmdmodify()
{
    return cmd_modifhook; 
}

/*
 * Function name:   modify_command
 * Description:	    This function is called from the master when modifying a
 *                  command.
 * Arguments:       cmd: The commandstring
 * Returns:         cmd if no modify object was defined,
 *                  otherwise the modify_command(cmd) in the modify object.
 */
public string
modify_command(string cmd)
{
    if (cmd[0] == '$')
	return extract(cmd, 1);
    if (cmd_modifhook)
	return cmd_modifhook->cmdmodify(cmd);
    return cmd;
}


/*
 * Function name: quit
 * Description:	  The standard routine when quitting.
 * Returns:	  1 / 0
 */
nomask int
quit()
{
    object *inv;
    int    il, loc;

    save_me(0);

    write("Saving " + query_name() + ".\n");
    say( ({ METNAME + " leaves the game.\n",
        TART_NONMETNAME + " leaves the game.\n",
        ""}) );

    loc = check_recover_loc();
    inv = all_inventory(this_object());
    for (il = 0; il < sizeof(inv); il++)
    {
	if (loc == 1 && inv[il]->check_recoverable() == 1)
	{
	    inv[il]->remove_object();
	    if (inv[il])
		destruct(inv[il]);
	}
	else if (!inv[il]->query_prop(OBJ_M_NO_DROP) && !query_wiz_level())
	    command("drop " + lower_case((string) inv[il]->query_name()));
	else
	{
	    inv[il]->remove_object();
	    if (inv[il])
		destruct(inv[il]);
	}
    }

    SECURITY->notify(this_object(), this_object()->query_real_name(), 0, 0);
    this_object()->remove_object();
    return 1;
}

/*
 * Function name:   save_character
 * Description:     Saves all internal variables of a character to disk
 * Returns:         Always 1
 */
public int
save_character() 
{
    save_me(1);
    write("Ok.\n");
    return 1;
}

/*
 * Function name:   autosave_me
 * Description:     Autosave all internal variables of a character to disk
 *                  every five minutes. Works only for non-wizards.
 * Returns:         Always 1
 */
public int
autosave_me() 
{
    int p;

    if (!query_wiz_level())
    {
	save_me(1);
	/* 
	 * Idle players shouldn't be interested in this. 
	 * If they are, it's probably just in order to remain idle.
	 */
	if (interactive() && query_idle(this_object()) < 180)
	    tell_object(this_object(), "Autosave.\n");
	p = query_prop(PLAYER_I_AUTOSAVE);
	if (p)
	    call_out("autosave_me", p * 60);
	else
	    call_out("autosave_me", 5 * 60);
    }
	
    return 1;
}

/*
 * Function name:   save_me
 * Description:     Save all internal variables of a character to disk.
 * Arguments:       value_items: If 0, set total value of money to 0,
 *                               otherwise calculate the total value of all the
 *                               stuff a character carries.
 */
public nomask void
save_me(int value_items)
{
    if (value_items)
	set_tot_value(compute_values(deep_inventory(this_object())));
    else
	set_tot_value(0);
    /* 
     * Do some queries to make certain time-dependent 
     * vars are updated properly.
     */
    query_mana();
    query_fatigue();
    query_hp();
    query_stuffed();
    query_soaked();
    query_intoxicated();
    query_headache();
    query_age();
    compute_auto_str();
    compute_recover_str(value_items);
    seteuid(0);
    SECURITY->save_player();
    seteuid(getuid(this_object()));
}

/*
 * Function name:   compute_values
 * Description:     Recursively compute the values of the given list of
 *                  objects. If the objects contain objects with a value
 *                  that value is also taken into account. Only 2/3 of the
 *		    actual value will be stored. Also, coins aren't included
 *		    in the value summed up. Don't figure objects that can
 *		    be recovered into this list.
 * Arguments:       ob_list: The list of objects to sum up.
 */
public nomask int
compute_values(object *ob_list)
{
    int v, i;

    if (!pointerp(ob_list))
	return 0;

    for (v = 0, i = 0; i < sizeof(ob_list); i++) 
    {
	int tmp;

	if (ob_list[i]->query_recover())
	    continue;
	if (function_exists("create_heap", ob_list[i]) == "/std/coins")
	    continue;
	tmp = 2 * ob_list[i]->query_prop(OBJ_I_VALUE) / 3;
	if (tmp > 1000)
	    tmp = 1000;
	v += tmp;
	if (ob_list[i] && all_inventory(ob_list[i]))
	    v += compute_values(all_inventory(ob_list[i]));
    }
    return v;
}

/*
 * Function name:   compute_auto_string
 * Description:     Walk through the inventory and check all the objects for
 *                  the function query_auto_load(). Constructs an array with
 *                  all returned strings. query_auto_load() should return
 *                  a string of the form "<file>:<argument>".
 */
static nomask void
compute_auto_str()
{
    object *ob;
    string str, *auto;
    int i;

    auto = ({ });
    ob = all_inventory(this_object());
    for (i = 0; i < sizeof(ob); i++)
    {
	str = (string) ob[i]->query_auto_load();
	if (!str)
	    continue;
	auto += ({ str });
    }
    set_auto_load(auto);
}

/*
 * Function name:   compute_recover_str
 * Description:     Walk through the inventory and check all the objects for
 *                  the recover property.
 * Arguments:       verb - be verbose 1, or not 0.
 */
static nomask void
compute_recover_str(int verb)
{
    object *ob;
    string str, *recover;
    int i, loc;

    recover = ({ });

    loc = check_recover_loc();
    ob = all_inventory(this_object());
    for (i = 0; i < sizeof(ob); i++)
    {
	if ((loc || verb) && ob[i]->check_recoverable() == 1)
	{
	    if (catch(str = (string)ob[i]->query_recover()))
		continue;
	    recover += ({ str });
	    if (loc)
		write("The " + ob[i]->short() + " glows briefly.\n");
	}
    }
    set_recover_list(recover);
}

/*
 * Function name: check_recover_loc
 * Description:   This function checks if the player is standing on a spot
 *                where recover may happend
 * Returns:       1 / 0 depending on outcome.
 */
nomask int
check_recover_loc()
{
    object tmp;
    int i;
    string *list, env;

    /* Check for armageddon */
    if (SECURITY->query_memory_failure() == 1)
	return 1;

    /* Check for recoverable surroundings */
    tmp = environment(this_object());
    if (tmp && objectp(tmp))
        env = file_name(tmp);
    else
	return 0;

    list = SECURITY->query_list_temp_start();
    list += SECURITY->query_list_def_start();
    for (i = 0 ; i < sizeof(list) ; i++)
    {
	if (env == list[i])
	{
	    return 1;
	    break;
	}
    }
    return 0;
}

/*
 * Function name:   show_age
 * Description:     Make a readable string of the number returned by the
 *                  function query_age(). 
 * Returns:         The age string, e.g.: "1 day 5 minutes 37 seconds".
 */
public string
show_age()
{
    int i, j;
    string str;

    str = "";
    i = query_age();
    if (j = i / 43200)
    {
        if (j > 1)
            str += " " + j + " days";
        else
            str += " 1 day";
        i = i - j * 43200;
    }
    if (j = i / 1800)
    {
        if (j > 1)
            str += " " + j + " hours";
        else
            str += " 1 hour";
        i = i  - j * 1800;
    }
    if (j = i / 30)
    {
        if (j > 1)
            str += " " + j + " minutes";
        else
            str += " 1 minute";
        i = i - (i / 30) * 30;
    }
    if ((j = i * 2) > 1)
        str += " " + i * 2 + " seconds";
    else if (j == 1)
        str += " 1 second";

    return str;
}

/*
 * Function name:   toggle_brief
 * Description:     Change the way a player looks at the world; brief or
 *                  verbose. Brief mode only shows the short descriptions
 *                  of the rooms when you walk through them, verbose mode
 *                  shows the long description.
 * Returns:         Always 1.
 */
public int
toggle_brief()
{
    set_brief(!query_brief());
    if (query_brief())
        write("Brief mode.\n");
    else
        write("Verbose mode.\n");
    return 1;
}

/*
 * This one is called when the player wants to change his password.
 */
static string password2;

/*
 * Function name:   change_password
 * Description:     Allow a player to change his old password into a new
 *                  one.
 * Arguments:       str: The old password
 * Returns:         Always 1.
 */
static nomask int
change_password(string str)
{
    write("Old password: ");
    input_to("change_password_old", 1);
    return 1;
}

/*
 * Function name:   change_password_old
 * Description:     Takes and checks the old password
 * Arguments:       str: The given password
 */
static nomask void 
change_password_old(string str)
{
    string password;

    password = query_password();

    if ((strlen(str) != 0 || strlen(password) != 0) &&
	password != crypt(str, password)) 
    {
	write("Wrong old password.\n");
	return;
    }
    password2 = 0;
    input_to("change_password2", 1);
    write("New password: ");
    return;
}

/*
 * Function name:   change_password2
 * Description:     This function is called by change_password to catch the
 *                  new password entered by the player. Calls itself again
 *                  to verify the new entered password.
 * Arguments:       str: The new password
 */
static nomask void
change_password2(string str)
{
    if (!strlen(str))
    {
	write("Password not changed.\n");
	return;
    }
    if (password2 == 0)
    {
	password2 = str;
	input_to("change_password2", 1);
	write("Again: ");
	return;
    }
    if (password2 != str)
    {
	write("Wrong! Password not changed.\n");
	return;
    }
    set_password(crypt(password2, 0));	/* Generate new seed */
    password2 = 0;
    write("Password changed.\n");
}

/*
 * Function name:   set_email
 * Description:     Set a new or ask the old E-mail address of a player.
 * Arguments:       str: If no string is given, show the old E-mail address
 *                       else set the E-mail to str.
 * Returns:         Always 1.
 */
static int
set_email(string str)
{
    if (!str)
    {
	write("Your official email address is: " + 
	      query_mailaddr() + "\n");
	return 1;
    }
    set_mailaddr(str);
    write("Changed your email address.\n");
    return 1;
}

==> Mudlib-CD.00.30.tar.gz: lib/std/player/cmd_sec.c <==
/*
 * player/cmd_sec.c
 *
 * This is a subpart of player_sec.c
 *
 * Some standard commands that should always exist are defined here.
 * This is also the place for the quicktyper command hook.
 *
 * This file is inherited by player_sec.c
 *
 * Revision history:
 *
 * 25-07-1993 /Mercade : added deep_inventory in compute_auto_string and
 *                       compute_recover_str
 */
#include "/sys/stdproperties.h"
#include "/secure/std.h"
#include "/sys/macros.h"

/*
 * Prototypes
 */
void save_me(int value_items);
int compute_values(object *ob_list);
void compute_auto_str();
void compute_recover_str(int verb);
string show_age();
nomask int check_recover_loc();

static 	object	cmd_modifhook;		/* Object to use for modif. of cmds */

static nomask void
cmd_sec_reset()
{
    add_action("quit", "quit");
    add_action("save_character", "save");
    add_action("toggle_brief", "brief");
    add_action("change_password", "password");
    add_action("set_email", "email");
    call_out("autosave_me", 5 * 60);
}

/*
 * Function name:   set_cmdmodify
 * Description:	    Decide what object to use for command modification. The
 *                  object should define the function modify_command(str).
 *                  If an object is set, commands that are not starting with
 *                  a '$' will be passed to that objects modifycmd function.
 * Arguments:       ob: object that defines the command modification function
 */
public void
set_cmdmodify(object ob)
{
    if (geteuid(ob) != geteuid(this_object()) && geteuid(ob) !=
		SECURITY->get_root_uid())
	return;

    cmd_modifhook = ob;
}

/*
 * Function name:   query_cmdmodify
 * Description:	    Returns the object that is used for command modification
 * Returns:         0 if no modify object was defined,
 *                  the modify object otherwise
 */
public object
query_cmdmodify()
{
    return cmd_modifhook; 
}

/*
 * Function name:   modify_command
 * Description:	    This function is called from the master when modifying a
 *                  command.
 * Arguments:       cmd: The commandstring
 * Returns:         cmd if no modify object was defined,
 *                  otherwise the modify_command(cmd) in the modify object.
 */
nomask public string
modify_command(string cmd)
{
    if (cmd[0] == '$')
	return extract(cmd, 1);
    if (cmd_modifhook)
	return cmd_modifhook->cmdmodify(cmd);
    return cmd;
}

/*
 * Function name: quit
 * Description:	  The standard routine when quitting.
 * Returns:	  1 / 0
 */
nomask int
quit()
{
    object *inv;
    int    il, loc;

    save_me(0);

    write("Saving " + query_name() + ".\n");
    say( ({ METNAME + " leaves the game.\n",
        TART_NONMETNAME + " leaves the game.\n",
        ""}) );

    loc = check_recover_loc();
    inv = all_inventory(this_object());
    for (il = 0; il < sizeof(inv); il++)
    {
	if (loc == 1 && inv[il]->check_recoverable() == 1)
	{
	    inv[il]->remove_object();
	    if (inv[il])
		MASTER->do_debug("destroy", inv[il]);
	}
	else if (!inv[il]->query_prop(OBJ_M_NO_DROP) && !query_wiz_level())
	    command("drop " + lower_case((string) inv[il]->query_name()));
	else
	{
	    inv[il]->remove_object();
	    if (inv[il])
		MASTER->do_debug("destroy", inv[il]);
	}
    }

    SECURITY->notify(this_object(), this_object()->query_real_name(), 0, 0);
    this_object()->remove_object();
    return 1;
}

/*
 * Function name:   save_character
 * Description:     Saves all internal variables of a character to disk
 * Returns:         Always 1
 */
public int
save_character() 
{
    save_me(1);
    write("Ok.\n");
    return 1;
}

/*
 * Function name:   autosave_me
 * Description:     Autosave all internal variables of a character to disk
 *                  every five minutes. Works only for non-wizards.
 * Returns:         Always 1
 */
public int
autosave_me() 
{
    int p;

    if (!query_wiz_level())
    {
	save_me(1);
	/* 
	 * Idle players shouldn't be interested in this. 
	 * If they are, it's probably just in order to remain idle.
	 */
	if (interactive() && query_idle(this_object()) < 180)
	    tell_object(this_object(), "Autosave.\n");
	p = query_prop(PLAYER_I_AUTOSAVE);
	if (p)
	    call_out("autosave_me", p * 60);
	else
	    call_out("autosave_me", 5 * 60);
    }
	
    return 1;
}

/*
 * Function name:   save_me
 * Description:     Save all internal variables of a character to disk.
 * Arguments:       value_items: If 0, set total value of money to 0,
 *                               otherwise calculate the total value of all the
 *                               stuff a character carries.
 */
public nomask void
save_me(int value_items)
{
    if (value_items)
	set_tot_value(compute_values(deep_inventory(this_object())));
    else
	set_tot_value(0);
    /* 
     * Do some queries to make certain time-dependent 
     * vars are updated properly.
     */
    query_mana();
    query_fatigue();
    query_hp();
    query_stuffed();
    query_soaked();
    query_intoxicated();
    query_headache();
    query_age();
    compute_auto_str();
    compute_recover_str(value_items);
    seteuid(0);
    SECURITY->save_player();
    seteuid(getuid(this_object()));
}

/*
 * Function name:   compute_values
 * Description:     Recursively compute the values of the given list of
 *                  objects. If the objects contain objects with a value
 *                  that value is also taken into account. Only 2/3 of the
 *		    actual value will be stored. Also, coins aren't included
 *		    in the value summed up. Don't figure objects that can
 *		    be recovered into this list.
 * Arguments:       ob_list: The list of objects to sum up.
 */
public nomask int
compute_values(object *ob_list)
{
    int v, i;

    if (!pointerp(ob_list))
	return 0;

    for (v = 0, i = 0; i < sizeof(ob_list); i++) 
    {
	int tmp;

	if (ob_list[i]->query_recover() || ob_list[i]->query_auto_load())
	    continue;
	if (function_exists("create_heap", ob_list[i]) == "/std/coins")
	    continue;
	tmp = 2 * ob_list[i]->query_prop(OBJ_I_VALUE) / 3;
	if (tmp > 1000)
	    tmp = 1000;
	v += tmp;
	if (ob_list[i] && all_inventory(ob_list[i]))
	    v += compute_values(all_inventory(ob_list[i]));
    }
    return v;
}

/*
 * Function name:   compute_auto_string
 * Description:     Walk through the inventory and check all the objects for
 *                  the function query_auto_load(). Constructs an array with
 *                  all returned strings. query_auto_load() should return
 *                  a string of the form "<file>:<argument>".
 */
static nomask void
compute_auto_str()
{
    object *ob;
    string str, *auto;
    int i;

    auto = ({ });
    ob = deep_inventory(this_object());
    for (i = 0; i < sizeof(ob); i++)
    {
	str = (string) ob[i]->query_auto_load();
	if (!str)
	    continue;
	auto += ({ str });
    }
    set_auto_load(auto);
}

/*
 * Function name:   compute_recover_str
 * Description:     Walk through the inventory and check all the objects for
 *                  the recover property.
 * Arguments:       verb - be verbose 1, or not 0.
 */
static nomask void
compute_recover_str(int verb)
{
    object *ob;
    string str, *recover;
    int i, loc;

    recover = ({ });

    loc = check_recover_loc();
    ob = deep_inventory(this_object());
    for (i = 0; i < sizeof(ob); i++)
    {
	if ((loc || verb) && ob[i]->check_recoverable() == 1)
	{
	    if (catch(str = (string)ob[i]->query_recover()))
		continue;
	    recover += ({ str });
	    if (loc)
		write("The " + ob[i]->short() + " glows briefly.\n");
	}
    }
    set_recover_list(recover);
}

/*
 * Function name: check_recover_loc
 * Description:   This function checks if the player is standing on a spot
 *                where recover may happend
 * Returns:       1 / 0 depending on outcome.
 */
nomask int
check_recover_loc()
{
    object tmp;
    int i;
    string *list, env;

    /* Check for armageddon */
    if (SECURITY->query_memory_failure() == 1)
	return 1;

    /* Check for recoverable surroundings */
    tmp = environment(this_object());
    if (tmp && objectp(tmp))
        env = file_name(tmp);
    else
	return 0;

    list = SECURITY->query_list_temp_start();
    list += SECURITY->query_list_def_start();
    for (i = 0 ; i < sizeof(list) ; i++)
    {
	if (env == list[i])
	{
	    return 1;
	    break;
	}
    }
    return 0;
}

/*
 * Function name:   show_age
 * Description:     Make a readable string of the number returned by the
 *                  function query_age(). 
 * Returns:         The age string, e.g.: "1 day 5 minutes 37 seconds".
 */
public string
show_age()
{
    int i, j;
    string str;

    str = "";
    i = query_age();
    if (j = i / 43200)
    {
        if (j > 1)
            str += " " + j + " days";
        else
            str += " 1 day";
        i = i - j * 43200;
    }
    if (j = i / 1800)
    {
        if (j > 1)
            str += " " + j + " hours";
        else
            str += " 1 hour";
        i = i  - j * 1800;
    }
    if (j = i / 30)
    {
        if (j > 1)
            str += " " + j + " minutes";
        else
            str += " 1 minute";
        i = i - (i / 30) * 30;
    }
    if ((j = i * 2) > 1)
        str += " " + i * 2 + " seconds";
    else if (j == 1)
        str += " 1 second";

    return str;
}

/*
 * Function name:   toggle_brief
 * Description:     Change the way a player looks at the world; brief or
 *                  verbose. Brief mode only shows the short descriptions
 *                  of the rooms when you walk through them, verbose mode
 *                  shows the long description.
 * Returns:         Always 1.
 */
public int
toggle_brief()
{
    set_brief(!query_brief());
    if (query_brief())
        write("Brief mode.\n");
    else
        write("Verbose mode.\n");
    return 1;
}

/*
 * This one is called when the player wants to change his password.
 */
static string password2;

/*
 * Function name:   change_password
 * Description:     Allow a player to change his old password into a new
 *                  one.
 * Arguments:       str: The old password
 * Returns:         Always 1.
 */
static nomask int
change_password(string str)
{
    write("Old password: ");
    input_to("change_password_old", 1);
    return 1;
}

/*
 * Function name:   change_password_old
 * Description:     Takes and checks the old password
 * Arguments:       str: The given password
 */
static nomask void 
change_password_old(string str)
{
    string password;

    password = query_password();

    if ((strlen(str) != 0 || strlen(password) != 0) &&
	password != crypt(str, password)) 
    {
	write("Wrong old password.\n");
	return;
    }
    password2 = 0;
    input_to("change_password2", 1);
    write("New password: ");
    return;
}

/*
 * Function name:   change_password2
 * Description:     This function is called by change_password to catch the
 *                  new password entered by the player. Calls itself again
 *                  to verify the new entered password.
 * Arguments:       str: The new password
 */
static nomask void
change_password2(string str)
{
    if (!strlen(str))
    {
	write("Password not changed.\n");
	return;
    }
    if (password2 == 0)
    {
	password2 = str;
	input_to("change_password2", 1);
	write("Again: ");
	return;
    }
    if (password2 != str)
    {
	write("Wrong! Password not changed.\n");
	return;
    }
    set_password(crypt(password2, 0));	/* Generate new seed */
    password2 = 0;
    write("Password changed.\n");
}

/*
 * Function name:   set_email
 * Description:     Set a new or ask the old E-mail address of a player.
 * Arguments:       str: If no string is given, show the old E-mail address
 *                       else set the E-mail to str.
 * Returns:         Always 1.
 */
static int
set_email(string str)
{
    if (!str)
    {
	write("Your official email address is: " + 
	      query_mailaddr() + "\n");
	return 1;
    }
    set_mailaddr(str);
    write("Changed your email address.\n");
    return 1;
}
